@@ -5,7 +5,7 @@ no strict;
 no warnings;
 use DBI;
 
-our $VERSION = "0.05";
+our $VERSION = "0.06";
 our $trace = 1;
 our $path = "STDERR";
 our $array;
@@ -15,101 +15,69 @@ our @queries;
 my $orig_execute = \&DBI::st::execute;
 *DBI::st::execute = sub {
     my ($sth, @args) = @_;
-    my $time1 = time();
-    my $retval = eval {$orig_execute->($sth, @args)};
-    my $error = $@;
-    my $time2 = time();
-    log_("execute", $time1, $time2, $sth->{Database}, $sth->{Statement}, \@args);
-    die $error if $error;
+    log_("execute", $sth->{Database}, $sth->{Statement}, \@args);
+    my $retval = $orig_execute->($sth, @args);
     return $retval;
 };
 
 my $orig_selectall_arrayref = \&DBI::db::selectall_arrayref;
 *DBI::db::selectall_arrayref = sub {
     my ($dbh, $query, $yup, @args) = @_;
-    my $time1 = time();
-    my $retval = eval {$orig_selectall_arrayref->($dbh, $query, $yup, @args)};
-    my $error = $@;
-    my $time2 = time();
-    log_("selectall_arrayref", $time1, $time2, $dbh, $query, \@args);
-    die $error if $error;
+    log_("selectall_arrayref", $dbh, $query, \@args);
+    my $retval = $orig_selectall_arrayref->($dbh, $query, $yup, @args);
     return $retval;
 };
 
 my $orig_selectcol_arrayref = \&DBI::db::selectcol_arrayref;
 *DBI::db::selectcol_arrayref = sub {
     my ($dbh, $query, $yup, @args) = @_;
-    my $time1 = time();
-    my $retval = eval {$orig_selectcol_arrayref->($dbh, $query, $yup, @args)};
-    my $error = $@;
-    my $time2 = time();
-    log_("selectcol_arrayref", $time1, $time2, $dbh, $query, \@args);
-    die $error if $error;
+    log_("selectcol_arrayref", $dbh, $query, \@args);
+    my $retval = $orig_selectcol_arrayref->($dbh, $query, $yup, @args);
     return $retval;
 };
 
 my $orig_selectrow_arrayref = \&DBI::db::selectrow_arrayref;
 *DBI::db::selectall_hashref = sub {
     my ($dbh, $query, $yup, @args) = @_;
-    my $time1 = time();
-    my $retval = eval {$orig_selectall_hashref->($dbh, $query, $yup, @args)};
-    my $error = $@;
-    my $time2 = time();
-    log_("selectall_hashref", $time1, $time2, $dbh, $query, \@args);
-    die $error if $error;
+    log_("selectall_hashref", $dbh, $query, \@args);
+    my $retval = $orig_selectall_hashref->($dbh, $query, $yup, @args);
     return $retval;
 };
 
 my $orig_selectrow_array = \&DBI::db::selectrow_array;
 *DBI::db::selectrow_arrayref = sub {
     my ($dbh, $query, $yup, @args) = @_;
-    my $time1 = time();
-    my $retval = eval {$orig_selectrow_arrayref->($dbh, $query, $yup, @args)};
-    my $error = $@;
-    my $time2 = time();
-    log_("selectrow_arrayref", $time1, $time2, $dbh, $query, \@args);
-    die $error if $error;
+    log_("selectrow_arrayref", $dbh, $query, \@args);
+    my $retval = $orig_selectrow_arrayref->($dbh, $query, $yup, @args);
     return $retval;
 };
 
 my $orig_selectrow_hashref = \&DBI::db::selectrow_hashref;
 *DBI::db::selectrow_array = sub {
     my ($dbh, $query, $yup, @args) = @_;
-    my $time1 = time();
-    my $retval = eval {$orig_selectrow_array->($dbh, $query, $yup, @args)};
-    my $error = $@;
-    my $time2 = time();
-    log_("selectrow_array", $time1, $time2, $dbh, $query, \@args);
-    die $error if $error;
+    log_("selectrow_array", $dbh, $query, \@args);
+    my $retval = $orig_selectrow_array->($dbh, $query, $yup, @args);
     return $retval;
 };
 
 my $orig_selectcol_arrayref = \&DBI::db::selectcol_arrayref;
 *DBI::db::selectrow_hashref = sub {
     my ($dbh, $query, $yup, @args) = @_;
-    my $time1 = time();
-    my $retval = eval {$orig_selectrow_hashref->($dbh, $query, $yup, @args)};
-    my $error = $@;
-    my $time2 = time();
-    log_("selectrow_hashref", $time1, $time2, $dbh, $query, \@args);
-    die $error if $error;
+    log_("selectrow_hashref", $dbh, $query, \@args);
+    my $retval = $orig_selectrow_hashref->($dbh, $query, $yup, @args);
     return $retval;
 };
 
 my $orig_do = \&DBI::db::do;
 *DBI::db::do = sub {
     my ($dbh, $query, $yup, @args) = @_;
-    my $time1 = time();
-    my $retval = eval {$orig_do->($dbh, $query, $yup, @args)};
-    my $error = $@;
-    my $time2 = time();
-    log_("do", $time1, $time2, $dbh, $query, \@args);
-    die $error if $error;
+    log_("do", $dbh, $query, \@args);
+    my $retval = $orig_do->($dbh, $query, $yup, @args);
     return $retval;
 };
 
 sub log_ {
-    my ($name, $time1, $time2, $dbh, $query, $args) = @_;
+    my ($name, $dbh, $query, $args) = @_;
     my $i = 0;
     my @callers;
     while (my @caller = caller($i++)) {
@@ -118,8 +86,7 @@ sub log_ {
     # subs like selectall_arrayref will call execute within it, we don't
     # want to log the same query twice
     return if (grep {$_->[0] eq "DBI::Log"} @callers) > 1;
-    my $diff = $time2 - $time1;
-    my $info = "-- " . scalar(localtime($time1)) . " taking $diff seconds\n";
+    my $info = "-- " . scalar(localtime()) . "\n";
     $i = 0;
     for my $caller (@callers) {
         my ($package, $file, $line, $sub) = @$caller;
@@ -129,12 +96,11 @@ sub log_ {
         $info .= "-- $sub $file $line\n";
         last if !$trace;
     }
-    $i = 0;
-    if ($dbh) {
-        $query =~ s/\?/$dbh->quote($args->[$i++])/eg;
-    }
-
     open_log();
+    my $i = 0;
+    if ($dbh) {
+        $query =~ s{\?}{$dbh->quote($args->[$i++])}eg;
+    }
     if ($fh) {
         print $fh "$info$query\n\n";
     }
@@ -152,17 +118,22 @@ sub open_log {
     elsif ($path eq "STDOUT") {
         $fh = \*STDOUT;
     }
-    else {
+    elsif ($path =~ m{^~/}) {
         my $home = (getpwuid($<))[7];
         $path =~ s{^~/}{$home/};
         open $fh, ">>", $path or die "Can't open $path: $!";
     }
+    else {
+        open $fh, ">>", $path or die "Can't open $path: $!";
+    }
 }
 
 1;
 
 __END__
 
+=encoding utf8
+
 =head1 NAME
 
 DBI::Log - Log all DBI queries
@@ -206,7 +177,10 @@ might be highlighted. This is what the output may look like:
 
 Each query in the log is prepended with the date, the time it took
 to run, and a stack trace. You can disable the stack trace by setting
-$DBI::Log::trace to a false value.
+$DBI::Log::trace to a false value or by setting the C<DBI_LOG_NOTRACE>
+environment variable (useful when running from the command line):
+
+    DBI_LOG_NOTRACE=1 perl -MDBI::Log program.pl
 
 You can set $DBI::Log::array to a true value and then all queries
 will end up in @DBI::Log::queries.
@@ -229,6 +203,10 @@ L<https://github.com/zorgnax/dbilog>
 
 Jacob Gelbman, E<lt>gelbman@gmail.comE<gt>
 
+=head1 CONTRIBUTORS
+
+Árpád Szász, E<lt>arpad.szasz@plenum.roE<gt>
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2015 by Jacob Gelbman
